// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Fully.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_FULLSCREEN, OnViewFullScreen)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FULLSCREEN, OnUpdateViewFullScreen)
	ON_WM_GETMINMAXINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pWndFullScreen = NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::EscapeFullScreen()
{
	if (m_pWndFullScreen != NULL)
		OnViewFullScreen();
}


void CMainFrame::OnViewFullScreen() 
{
	TRACE("m_wndStatusBar == %d\n", m_wndStatusBar.IsWindowVisible());
	TRACE("m_wndToolBar   == %d\n", m_wndToolBar.IsWindowVisible());

	if (m_pWndFullScreen != NULL)
	{
		// If we are returning the app to the Maximized state, SetWindowPlacement()
		// draws the entire app twice - once to its "Normal" state, then once to
		// the maximized state.  The LockWindowUpdate() calls suppress the extra
		// redraw.

		ShowControlBar(&m_wndToolBar, m_bHadToolBar, FALSE);
		ShowControlBar(&m_wndStatusBar, m_bHadStatusBar, FALSE);

		m_pWndFullScreen->LockWindowUpdate();
		m_pWndFullScreen->SetWindowPlacement(&m_PreviousPlace);
		m_pWndFullScreen->UnlockWindowUpdate();

		LockWindowUpdate();
		SetWindowPlacement(&m_MainPreviousPlace);
		UnlockWindowUpdate();
		m_pWndFullScreen = NULL;
	}
	else
	{
		m_pWndFullScreen = MDIGetActive();
		ASSERT(m_pWndFullScreen != NULL);

		m_bHadToolBar = m_wndToolBar.IsWindowVisible();
		m_bHadStatusBar = m_wndStatusBar.IsWindowVisible();
		ShowControlBar(&m_wndToolBar, FALSE, FALSE);
		ShowControlBar(&m_wndStatusBar,FALSE, FALSE);

		m_pWndFullScreen->GetWindowPlacement(&m_PreviousPlace);
		GetWindowPlacement(&m_MainPreviousPlace);

		CRect rectDesktop;
		if (!::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktop, 0))
	 		::GetWindowRect(::GetDesktopWindow(), &rectDesktop);
		else
		{
			rectDesktop.right = ::GetSystemMetrics(SM_CXSCREEN);
			rectDesktop.bottom = ::GetSystemMetrics(SM_CYSCREEN);
			rectDesktop.right -= rectDesktop.left;
			rectDesktop.bottom -= rectDesktop.top;
			rectDesktop.left = -rectDesktop.left;
			rectDesktop.top = -rectDesktop.top;
		}

		TRACE("Going to (%d,%d) - (%d,%d)\n", rectDesktop.left, rectDesktop.top,
			rectDesktop.right, rectDesktop.bottom);

		CRect rectDesired = rectDesktop;
		::AdjustWindowRect(&rectDesktop, GetStyle(), TRUE);

		TRACE("Adjusted to (%d,%d) - (%d,%d)\n", rectDesktop.left, rectDesktop.top,
			rectDesktop.right, rectDesktop.bottom);

		rectDesktop.bottom += ::GetSystemMetrics(SM_CYBORDER);	// See "Ask Dr. GUI #10"
		m_FullScreenPlace = rectDesktop;
		m_MainFullScreenPlace = rectDesktop;

		WINDOWPLACEMENT wpNew = m_PreviousPlace;
		wpNew.showCmd =  SW_SHOWNORMAL;
		wpNew.rcNormalPosition = rectDesktop;

		LockWindowUpdate();
		SetWindowPlacement(&wpNew);
		UnlockWindowUpdate();

		TRACE("Before MDI Child to (%d,%d) - (%d,%d)\n", wpNew.rcNormalPosition.left, wpNew.rcNormalPosition.top,
			wpNew.rcNormalPosition.right, wpNew.rcNormalPosition.bottom);

		wpNew.rcNormalPosition = rectDesired;
		::AdjustWindowRect(&(wpNew.rcNormalPosition), m_pWndFullScreen->GetStyle(), FALSE);

		TRACE("MDI Child to (%d,%d) - (%d,%d)\n", wpNew.rcNormalPosition.left, wpNew.rcNormalPosition.top,
			wpNew.rcNormalPosition.right, wpNew.rcNormalPosition.bottom);

		m_pWndFullScreen->LockWindowUpdate();
		m_pWndFullScreen->SetWindowPlacement(&wpNew);
		m_pWndFullScreen->UnlockWindowUpdate();
	}
}

void CMainFrame::OnUpdateViewFullScreen(CCmdUI* pCmdUI) 
{
	CMDIChildWnd* pWnd = MDIGetActive();
	pCmdUI->Enable(pWnd != NULL);
	pCmdUI->SetCheck(m_pWndFullScreen != NULL);
}

void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if (m_pWndFullScreen != NULL)
	{
		lpMMI->ptMaxSize.y = 
 		lpMMI->ptMaxTrackSize.y = m_FullScreenPlace.Height();
		lpMMI->ptMaxSize.x = 
		lpMMI->ptMaxTrackSize.x = m_FullScreenPlace.Width();
	}
	else
		CMDIFrameWnd::OnGetMinMaxInfo(lpMMI);  
}

BOOL CMainFrame::GetFullScreenSize(CWnd* pWnd, POINT& pt) const
{
	if (m_pWndFullScreen != NULL && pWnd->m_hWnd == m_pWndFullScreen->m_hWnd)
	{
		pt.x = m_FullScreenPlace.Width();
		pt.y = m_FullScreenPlace.Height();
		pWnd->ScreenToClient(&pt);
	}

	TRACE("FullScreen Size is %d, %d\n", pt.x, pt.y);
	return (m_pWndFullScreen != NULL);
}

